@@ -30,6 +30,12 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: deviate.c,v $
+** Revision 1.25  2006/02/07 06:45:33  gerlof
+** Removed swap-counter.
+**
+** Revision 1.24  2006/01/30 09:13:33  gerlof
+** Extend memory counters (a.o. page scans).
+**
 ** Revision 1.23  2005/10/31 12:45:29  gerlof
 ** Support account-record version 3 (used by Mandriva).
 **
@@ -101,7 +107,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.23 2005/10/31 12:45:29 gerlof Exp $";
+static const char rcsid[] = "$Id: deviate.c,v 1.25 2006/02/07 06:45:33 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -243,8 +249,6 @@ deviatproc(struct pstat *aproc, int npresent,
 			subcount(curstat->mem.minflt, prestat.mem.minflt);
 		devstat->mem.majflt =
 			subcount(curstat->mem.majflt, prestat.mem.majflt);
-		devstat->mem.iosw   =
-			subcount(curstat->mem.iosw, prestat.mem.iosw);
 
 		devstat->net.tcpsnd =
 			subcount(curstat->net.tcpsnd, prestat.net.tcpsnd);
@@ -396,7 +400,6 @@ deviatproc(struct pstat *aproc, int npresent,
 
 		devstat->cpu.stime  = curstat->cpu.stime  - prestat.cpu.stime;
 		devstat->cpu.utime  = curstat->cpu.utime  - prestat.cpu.utime;
-		devstat->mem.iosw   = curstat->mem.iosw   - prestat.mem.iosw;
 		devstat->mem.minflt = curstat->mem.minflt - prestat.mem.minflt;
 		devstat->mem.majflt = curstat->mem.majflt - prestat.mem.majflt;
 		devstat->mem.shtext = 0;
@@ -420,8 +423,6 @@ deviatproc(struct pstat *aproc, int npresent,
 			devstat->dsk.wio   = 0;
 		if (devstat->dsk.wsz    < 0)
 			devstat->dsk.wsz   = 0;
-		if (devstat->mem.iosw   < 0)
-			devstat->mem.iosw  = 0;
 		if (devstat->mem.minflt < 0)
 			devstat->mem.minflt = 0;
 		if (devstat->mem.majflt < 0)
@@ -508,12 +509,16 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->mem.buffermem	= cur->mem.buffermem;
 	dev->mem.slabmem	= cur->mem.slabmem;
 	dev->mem.committed	= cur->mem.committed;
+	dev->mem.commitlim	= cur->mem.commitlim;
 	dev->mem.cachemem	= cur->mem.cachemem;
 	dev->mem.totswap	= cur->mem.totswap;
 	dev->mem.freeswap	= cur->mem.freeswap;
 
 	dev->mem.swouts		= subcount(cur->mem.swouts,  pre->mem.swouts);
 	dev->mem.swins		= subcount(cur->mem.swins,   pre->mem.swins);
+	dev->mem.pgscans	= subcount(cur->mem.pgscans, pre->mem.pgscans);
+	dev->mem.allocstall	= subcount(cur->mem.allocstall,
+				                         pre->mem.allocstall);
 
 	dev->net.ipin 		= subcount(cur->net.ipin,    pre->net.ipin);
 	dev->net.ipout 		= subcount(cur->net.ipout,   pre->net.ipout);
