@@ -40,7 +40,7 @@ struct pstat {
 		int	nthr;		/* number of threads in tgroup 	*/
 		char	name[PNAMLEN+1];/* process name string       	*/
 		char 	state;		/* process state ('E' = exited)	*/
-		long	excode;		/* process exit status		*/
+		int	excode;		/* process exit status		*/
 		time_t 	btime;		/* process start time (epoch)	*/
 		char	cmdline[CMDLEN+1];/* command-line string       	*/
 		int	future1;	/* reserved for future use	*/
@@ -73,7 +73,7 @@ struct pstat {
 
 	/* MEMORY STATISTICS						*/
 	struct mem {
-		count_t	iosw;		/* number of swaps	 	*/
+		count_t	future1;	/* reserved for future use	*/
 		count_t	minflt;		/* number of page-reclaims 	*/
 		count_t	majflt;		/* number of page-faults 	*/
 		count_t	shtext;		/* text     memory (Kb)         */
@@ -81,8 +81,8 @@ struct pstat {
 		count_t	rmem;		/* resident memory (Kb)		*/
 		count_t vgrow;		/* virtual  growth (Kb)    	*/
 		count_t rgrow;		/* resident growth (Kb)     	*/
-		count_t	future1;	/* reserved for future use	*/
 		count_t	future2;	/* reserved for future use	*/
+		count_t	future3;	/* reserved for future use	*/
 	} mem;
 
 	/* NETWORK STATISTICS						*/
