@@ -30,6 +30,9 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photosyst.c,v $
+** Revision 1.21  2006/01/30 09:14:16  gerlof
+** Extend memory counters (a.o. page scans).
+**
 ** Revision 1.20  2005/10/21 09:50:08  gerlof
 ** Per-user accumulation of resource consumption.
 **
@@ -93,7 +96,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.20 2005/10/21 09:50:08 gerlof Exp $";
+static const char rcsid[] = "$Id: photosyst.c,v 1.21 2006/01/30 09:14:16 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -237,7 +240,7 @@ photosyst(struct sstat *si)
 	*/
 	if ( (fp = fopen("vmstat", "r")) != NULL)
 	{
-		int	nrfields = 2;	/* number of fields to be filled */
+		int	nrfields = 9;	/* number of fields to be filled */
 
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL &&
 								nrfields > 0)
@@ -260,6 +263,20 @@ photosyst(struct sstat *si)
 				nrfields--;
 				continue;
 			}
+
+			if ( strcmp("allocstall", nam) == EQ)
+			{
+				si->mem.allocstall = cnts[0];
+				nrfields--;
+				continue;
+			}
+
+			if ( strncmp("pgscan_", nam, 7) == EQ)
+			{
+				si->mem.pgscans += cnts[0];
+				nrfields--;
+				continue;
+			}
 		}
 
 		fclose(fp);
@@ -287,7 +304,7 @@ photosyst(struct sstat *si)
 
 	if ( (fp = fopen("meminfo", "r")) != NULL)
 	{
-		int	nrfields = 8;	/* number of fields to be filled */
+		int	nrfields = 9;	/* number of fields to be filled */
 
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL && 
 								nrfields > 0)
@@ -380,6 +397,11 @@ photosyst(struct sstat *si)
 					si->mem.committed = cnts[0]*1024/pgsz;
 					nrfields--;
 				}
+			else	if (strcmp("CommitLimit:", nam) == EQ)
+				{
+					si->mem.commitlim = cnts[0]*1024/pgsz;
+					nrfields--;
+				}
 		}
 
 		fclose(fp);
