@@ -41,9 +41,9 @@ struct	memstat {
 
 	count_t	totswap;	/* number of pages in swap	*/
 	count_t	freeswap;	/* number of free swap pages	*/
-	count_t	future1;	/* reserved for future use	*/
-	count_t	future2;	/* reserved for future use	*/
-	count_t	future3;	/* reserved for future use	*/
+	count_t	pgscans;	/* number of page scans		*/
+	count_t	commitlim;	/* commit limit 		*/
+	count_t	allocstall;	/* try to free pages forced	*/
 };
 
 /************************************************************************/
