@@ -30,6 +30,13 @@
 ** --------------------------------------------------------------------------
 ** 
 ** $Log: showgeneric.c,v $
+** Revision 1.41  2006/04/03 05:42:35  gerlof
+** *** empty log message ***
+**
+** Revision 1.40  2006/02/07 08:28:26  gerlof
+** Improve screen-handling (less flashing) by exchanging clear()
+** by werase() (contribution Folkert van Heusden).
+**
 ** Revision 1.39  2005/11/04 14:16:16  gerlof
 ** Minor bug-solutions.
 **
@@ -152,7 +159,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showgeneric.c,v 1.39 2005/11/04 14:16:16 gerlof Exp $";
+static const char rcsid[] = "$Id: showgeneric.c,v 1.41 2006/04/03 05:42:35 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -178,8 +185,6 @@ static const char rcsid[] = "$Id: showgeneric.c,v 1.39 2005/11/04 14:16:16 gerlo
 #include "photosyst.h"
 #include "showgeneric.h"
 
-#define	MINLINES	12
-
 extern unsigned long	interval;
 extern int		deviatonly;
 extern char		rawreadflag;
@@ -193,6 +198,7 @@ static void	showhelp(int);
 static int	screen;     	/* boolean: screen control-codes needed */
 static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
+static int	avgval;		/* boolean: average values i.s.o. total */
 
 static int	(*curcompar)(const void *, const void *) = compcpu;
 static char	*sortorder = "CPU", showtype = MPROCGEN;
@@ -218,10 +224,11 @@ generic_samp(time_t curtime, int nsecs,
 {
 	static int	callnr = 0;
 
-	register int	i, curline=1, statline;
-	int		firstproc, plistsz, alistsz, killpid, killsig;
+	register int	i, curline, statline;
+	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
 	char		format1[16], format2[16];
+	char		*statmsg = NULL;
 	struct passwd 	*pwd;
 	struct syscap	syscap;
 
@@ -314,6 +321,13 @@ generic_samp(time_t curtime, int nsecs,
 				showtype  = MPROCARG;
 				break;
 
+			   case MAVGVAL:
+				if (avgval)
+					avgval=0;
+				else
+					avgval=1;
+				break;
+
 			   case MCUMUSER:
 				showtype  = MCUMUSER;
 				break;
@@ -392,102 +406,111 @@ generic_samp(time_t curtime, int nsecs,
 	*/
 	totalcap(&syscap, sstat, pstat, nact);
 
-        /*
-        ** prepare screen or file output for new sample
-        */
-        if (screen)
-                clear();
-        else
-                printf("\n\n");
-
-        /*
-        ** print general headerlines
-        */
-        convdate(curtime, format1);       /* convert date to ascii string   */
-        convtime(curtime, format2);       /* convert time to ascii string   */
-
-	if (screen)
-		attron(A_REVERSE);
-
-       	printg(genhdr, utsname.nodename, format1, format2, nsecs);
-
-	if (screen)
-		attroff(A_REVERSE);
-
 	/*
-	** print cumulative system- and user-time for all processes
+	** loop in which the system resources and the list of active
+	** processes is shown; the loop will be preempted by receiving
+	** a timer-signal or when the trigger-button is pressed.
 	*/
-	pricumproc(pstat, nact, nproc, nzomb, nexit);
-
-	curline++;
-
-	/*
-	** sort per-cpu       statistics on busy percentage
-	** sort per-disk      statistics on busy percentage
-	** sort per-interface statistics on transferred packages
-	*/
-	if (sstat->cpu.nrcpu > 1)
-		qsort(sstat->cpu.cpu, sstat->cpu.nrcpu,
-				sizeof sstat->cpu.cpu[0], cpucompar);
-
-	if (sstat->xdsk.nrxdsk > 1)
-		qsort(sstat->xdsk.xdsk, sstat->xdsk.nrxdsk,
-				sizeof sstat->xdsk.xdsk[0], diskcompar);
-
-	if (sstat->intf.nrintf > 1)
-		qsort(sstat->intf.intf, sstat->intf.nrintf,
-			  	sizeof sstat->intf.intf[0], intfcompar);
-
-	/*
-	** print other lines of system-wide statistics
-	*/
-	curline = prisyst(sstat, curline, nsecs, fixedhead,
-				maxcpulines, maxdsklines, maxintlines);
-
-	if (screen && curline+2 > LINES)
-	{
-		move(0, 0);
-		clrtobot();
-		move(0, 0);
-		printw("Not enough screen-lines available "
-			       "(need at least %d lines)\n", curline+2);
-		move(1, 0);
-		printw("Resize window or start atop with the -l flag to "
-		       "limit system-resources");
-
-		refresh();
-		sleep(4);
-		cleanstop(1);
-	}
-
-	statline = curline;
-
-	if (bootflag)
-	{
-		if (screen)
-			attron(A_BLINK);
-
-       		printg("                "
-		       "*** system and process activity since boot ***");
-
-		if (screen)
-			attroff(A_BLINK);
-	}
-
-	if (screen)
-		plistsz = LINES-curline-2;
-	else
-		plistsz = nact;
-
-	/*
-	** loop in which the list of active processes is shown;
-	** the loop will be preempted by receiving a timer-signal
- 	** or when the trigger-button is pressed.
-	*/
-	firstproc = 0;
-
 	while (1)
 	{
+		curline = 1;
+
+	        /*
+       	 	** prepare screen or file output for new sample
+        	*/
+        	if (screen)
+               	 	werase(stdscr);
+        	else
+                	printf("\n\n");
+
+        	/*
+        	** print general headerlines
+        	*/
+        	convdate(curtime, format1);       /* date to ascii string   */
+        	convtime(curtime, format2);       /* time to ascii string   */
+
+		if (screen)
+			attron(A_REVERSE);
+
+       		printg(genhdr, utsname.nodename, format1, format2, nsecs);
+
+		if (screen)
+			attroff(A_REVERSE);
+
+		/*
+		** print cumulative system- and user-time for all processes
+		*/
+		pricumproc(pstat, nact, nproc, nzomb, nexit, avgval, nsecs);
+
+		curline++;
+
+		/*
+		** sort per-cpu       statistics on busy percentage
+		** sort per-disk      statistics on busy percentage
+		** sort per-interface statistics on transferred packages
+		*/
+		if (sstat->cpu.nrcpu > 1)
+			qsort(sstat->cpu.cpu, sstat->cpu.nrcpu,
+		 	               sizeof sstat->cpu.cpu[0], cpucompar);
+
+		if (sstat->xdsk.nrxdsk > 1)
+			qsort(sstat->xdsk.xdsk, sstat->xdsk.nrxdsk,
+				       sizeof sstat->xdsk.xdsk[0], diskcompar);
+
+		if (sstat->intf.nrintf > 1)
+			qsort(sstat->intf.intf, sstat->intf.nrintf,
+			  	       sizeof sstat->intf.intf[0], intfcompar);
+
+		/*
+		** print other lines of system-wide statistics
+		*/
+		curline = prisyst(sstat, curline, nsecs, avgval, fixedhead,
+				maxcpulines, maxdsklines, maxintlines);
+
+		if (screen && curline+2 > LINES)
+		{
+			move(0, 0);
+			clrtobot();
+			move(0, 0);
+			printw("Not enough screen-lines available "
+			       "(need at least %d lines)\n", curline+2);
+			move(1, 0);
+			printw("Resize window or start atop with the -l flag"
+		       	       "to limit system-resources");
+
+			refresh();
+			sleep(4);
+			cleanstop(1);
+		}
+
+		statline = curline;
+
+		if (statmsg)
+		{
+			clrtoeol();
+			printg(statmsg);
+			statmsg = NULL;
+		}
+		else
+		{
+			if (bootflag)
+			{
+				if (screen)
+					attron(A_BLINK);
+
+       				printg("                *** system and "
+				       "process activity since boot ***");
+
+				if (screen)
+					attroff(A_BLINK);
+			}
+		}
+
+		if (screen)
+			plistsz = LINES-curline-2;
+		else
+			plistsz = nact;
+
 		/*
 		** if cumulative figures required, save the current
 		** list of processes and accumulate resource consumption
@@ -556,7 +579,7 @@ generic_samp(time_t curtime, int nsecs,
 			priproc(pstat, firstproc, nact, curline+2,
 			        firstproc/plistsz+1, (nact-1)/plistsz+1,
 			        sortorder, &syscap, showtype, 
-				&procsel, screen);
+				&procsel, screen, nsecs, avgval);
 		}
 
 		alistsz = nact;		/* preserve size of active list */
@@ -629,8 +652,8 @@ generic_samp(time_t curtime, int nsecs,
 			   case MSAMPPREV:
 				if (!rawreadflag)
 				{
-					printw("Only allowed when viewing "
-					       "raw file!");
+					statmsg = "Only allowed when viewing "
+					          "raw file!";
 					beep();
 					break;
 				}
@@ -702,8 +725,8 @@ generic_samp(time_t curtime, int nsecs,
 			   case MPROCDSK:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
-					printw("No kernel-patch installed; "
-					       "request ignored!");
+					statmsg = "No kernel-patch installed; "
+					          "request ignored!";
 					break;
 				}
 				showtype  = MPROCDSK;
@@ -718,8 +741,8 @@ generic_samp(time_t curtime, int nsecs,
 			   case MPROCNET:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
-					printw("No kernel-patch installed; "
-					       "request ignored!");
+					statmsg = "No kernel-patch installed; "
+					          "request ignored!";
 					break;
 				}
 				showtype  = MPROCNET;
@@ -758,9 +781,8 @@ generic_samp(time_t curtime, int nsecs,
 			   ** accumulated resource consumption per user
 			   */
 			   case MCUMUSER:
-				printw("Consumption per user; use '%c' to "
-				       "toggle between all/active processes",
-				       MALLPROC);
+				statmsg = "Consumption per user; use 'a' to "
+				          "toggle between all/active processes";
 
 				showtype  = MCUMUSER;
 				firstproc = 0;
@@ -770,9 +792,8 @@ generic_samp(time_t curtime, int nsecs,
 			   ** accumulated resource consumption per program
 			   */
 			   case MCUMPROC:
-				printw("Consumption per command; use '%c' to "
-				       "toggle between all/active processes",
-				       MALLPROC);
+				statmsg = "Consumption per program; use 'a' to "
+				          "toggle between all/active processes";
 
 				showtype  = MCUMPROC;
 				firstproc = 0;
@@ -785,16 +806,16 @@ generic_samp(time_t curtime, int nsecs,
 			   case MHELP2:
 				alarm(0);	/* stop the clock         */
 
-				move(statline, 0);
-				clrtobot();	/* blank the process-list */
+				move(1, 0);
+				clrtobot();	/* blank the screen */
 				refresh();
 
-				showhelp(statline+1);
+				showhelp(2);
 
 				move(statline, 0);
 
 				if (interval && !paused && !rawreadflag)
-					alarm(1); /* force new sample     */
+					alarm(3); /* force new sample     */
 
 				firstproc = 0;
 				break;
@@ -805,8 +826,8 @@ generic_samp(time_t curtime, int nsecs,
 			   case MKILLPROC:
 				if (rawreadflag)
 				{
-					printw("Not possible when viewing "
-					       "raw file!");
+					statmsg = "Not possible when viewing "
+					          "raw file!";
 					beep();
 					break;
 				}
@@ -823,11 +844,9 @@ generic_samp(time_t curtime, int nsecs,
 					break;
 
 				   case 1:
-					printw("Sending signal to pid 1 not "
-					       "allowed!");
+					statmsg = "Sending signal to pid 1 not "
+					          "allowed!";
 					beep();
-					refresh();
-					sleep(3);	
 					break;
 
 				   default:
@@ -837,17 +856,14 @@ generic_samp(time_t curtime, int nsecs,
 
 					if ( kill(killpid, killsig) == -1)
 					{
-						printw("Not possible to send "
-					               "signal %d to pid %d!",
-							killsig, killpid);
+						statmsg = "Not possible to "
+						     "send signal to this pid!";
 						beep();
-						refresh();
-						sleep(3);	
 					}
 				}
 
 				if (!paused)
-					alarm(1); /* set short timer */
+					alarm(3); /* set short timer */
 
 				firstproc = 0;
 				break;
@@ -858,8 +874,8 @@ generic_samp(time_t curtime, int nsecs,
 			   case MINTERVAL:
 				if (rawreadflag)
 				{
-					printw("Not possible when viewing "
-					       "raw file!");
+					statmsg = "Not possible when viewing "
+					          "raw file!";
 					beep();
 					break;
 				}
@@ -873,15 +889,12 @@ generic_samp(time_t curtime, int nsecs,
 				if (interval)
 				{
 					if (!paused)
-						alarm(1); /* set short timer */
+						alarm(3); /* set short timer */
 				}
 				else
 				{
-					clrtoeol();
-					printw("No timer set; waiting for "
-					       "manual trigger ('%c').....",
-						MSAMPNEXT);
-					refresh();
+					statmsg = "No timer set; waiting for "
+					          "manual trigger ('t').....";
 				}
 
 				firstproc = 0;
@@ -931,13 +944,9 @@ generic_samp(time_t curtime, int nsecs,
 
 					if (u == 0)
 					{
+						statmsg = "No user-names "
+						    "matching this pattern!";
 						beep();
-						move(statline, 0);
-						clrtoeol();
-						printw("No user-names matching"
-						       " this pattern!");
-						refresh();
-						sleep(3);
 					}
 				}
 				else
@@ -945,10 +954,8 @@ generic_samp(time_t curtime, int nsecs,
 					procsel.userid[0] = USERSTUB;
 				}
 
-				move(statline, 0);
-
 				if (interval && !paused && !rawreadflag)
-					alarm(1);  /* set short timer */
+					alarm(3);  /* set short timer */
 
 				firstproc = 0;
 				break;
@@ -980,7 +987,7 @@ generic_samp(time_t curtime, int nsecs,
 				move(statline, 0);
 
 				if (interval && !paused && !rawreadflag)
-					alarm(1);  /* set short timer */
+					alarm(3);  /* set short timer */
 
 				firstproc = 0;
 				break;
@@ -1014,30 +1021,40 @@ generic_samp(time_t curtime, int nsecs,
 			   case MALLPROC:
 				if (rawreadflag)
 				{
-					printw("Process-list from raw file "
-					       "will be shown anyhow!");
+					statmsg = "Process-list from raw file "
+					          "will be shown anyhow!";
 					break;
 				}
 
 				if (deviatonly)
 				{
 					deviatonly=0;
-					printw("All processes will be "
-					       "shown/accumulated......");
+					statmsg = "All processes will be "
+					          "shown/accumulated......";
 				}
 				else
 				{
 					deviatonly=1;
-					printw("Only active processes will"
-					       " be shown/accumulated.....");
+					statmsg = "Only active processes will"
+					          " be shown/accumulated.....";
 				}
 
 				if (interval && !paused && !rawreadflag)
-					alarm(1);  /* set short timer */
+					alarm(3);  /* set short timer */
 
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** toggle average or total values
+			   */
+			   case MAVGVAL:
+				if (avgval)
+					avgval=0;
+				else
+					avgval=1;
+				break;
+
 			   /*
 			   ** system-statistics lines:
 			   **	         toggle fixed or variable
@@ -1046,14 +1063,14 @@ generic_samp(time_t curtime, int nsecs,
 				if (fixedhead)
 				{
 					fixedhead=0;
-					printw("Only active system-resources "
-					       "will be shown ......");
+					statmsg = "Only active system-resources"
+					          " will be shown ......";
 				}
 				else
 				{
 					fixedhead=1;
-					printw("Also inactive system-resources"
-					       " will be shown.....");
+					statmsg = "Also inactive "
+					  "system-resources will be shown.....";
 				}
 
 				firstproc = 0;
@@ -1082,7 +1099,7 @@ generic_samp(time_t curtime, int nsecs,
 					    maxintlines, statline);
 
 				if (interval && !paused && !rawreadflag)
-					alarm(1);  /* set short timer */
+					alarm(3);  /* set short timer */
 
 				firstproc = 0;
 				break;
@@ -1099,7 +1116,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** show version info
 			   */
 			   case MVERSION:
-				printw("%s", getversion());
+				statmsg = getversion();
 				break;
 
 			   /*
@@ -1122,8 +1139,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** handle screen resize
 			   */
 			   case KEY_RESIZE:
-				printg("Resizing to %d lines and %d columns",
-						LINES, COLS);
+				statmsg = "Window has been resized....";
 				break;
 
 			   /*
@@ -1373,6 +1389,8 @@ static struct helptext {
 								MPAUSE},
 	{"\t'%c' - fixate on static range of header-lines           (toggle)\n",
 								MSYSFIXED},
+	{"\t'%c' - show average-per-second i.s.o. total values      (toggle)\n",
+								MAVGVAL},
 	{"\t'%c' - limited lines for per-cpu, disk and interface resources\n",
 								MSYSLIMIT},
 	{"\t'%c' - kill a process (i.e. send a signal)\n",	MKILLPROC},
@@ -1490,6 +1508,8 @@ generic_usage(void)
 			MSYSFIXED);
 	printf("\t  -%c  limited lines for system-resources\n",
 			MSYSLIMIT);
+	printf("\t  -%c  show average-per-second i.s.o. total values\n",
+			MAVGVAL);
 }
 
 /*
@@ -1673,6 +1693,10 @@ do_flags(char *val)
 			deviatonly  = 0;
 			break;
 
+		   case MAVGVAL:
+			avgval=1;
+			break;
+
 		   case MSYSFIXED:
 			fixedhead=1;
 			break;
