@@ -29,6 +29,19 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: showlinux.c,v $
+** Revision 1.31  2006/02/07 08:38:49  gerlof
+** Swapped the zombie counter and exit counter in the PRC-line.
+**
+** Revision 1.30  2006/02/07 08:30:07  gerlof
+** Add possibility to show counters per second.
+** Ease parsing of output-lines by fixed number of columns per line.
+**
+** Revision 1.29  2006/01/30 09:24:12  gerlof
+** PRC-line: 'exits' modified to 'exit' to save space.
+**
+** Revision 1.28  2006/01/30 09:14:26  gerlof
+** Extend memory counters (a.o. page scans).
+**
 ** Revision 1.27  2005/11/04 14:16:45  gerlof
 ** Minor bug-solutions.
 **
@@ -119,7 +132,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.27 2005/11/04 14:16:45 gerlof Exp $";
+static const char rcsid[] = "$Id: showlinux.c,v 1.31 2006/02/07 08:38:49 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -153,31 +166,29 @@ extern int      osvers;
 /*
 ** format-lines for printing
 */
-char 	*prcline   	= "PRC | sys %s | user %s | #thr %s | #exits %s |"
-	       	     	  " #zombie %s |\n";
+char 	*prcline   	= "PRC | sys %s | user %s | #thr %s | #zombie %s |"
+	       	     	  " #exit %s |\n";
 /***************************************************************************/
 char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW USERNAME THR  "
 		     	  "ST EXC  S  %s CMD  %4d/%-4d\n";
-char 	*genproclina1	= "%5d %s %s %s %s %-8.8s %3d  %c%c      %c %3.0lf%% "
+char 	*genproclina1	= "%5d %s %s %s %s %-8.8s %3d  %c%c   -  %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*genprocline1	= "%s %s               %-8.8s      %c%c %3ld  "
+char 	*genprocline1	= "%s %s     0K     0K %-8.8s   0  %c%c %3ld  "
 			  "E %3.0lf%% <%.12s>\n";
 
 char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW RDDSK WRDSK "
 		     	  "RNET SNET S  %s CMD  %4d/%-4d\n";
 char 	*genproclina2	= "%5d %s %s %s %s %s %s %s %s %c %3.0lf%% %.14s\n";
-char 	*genprocline2	= "%s %s               (r&w %s)        "
-			  "  E %3.0lf%% <%.12s>\n";
+char 	*genprocline2	= "%s %s     0K     0K (r&w %s)   -    "
+			  "- E %3.0lf%% <%.12s>\n";
 char 	*genprocline2p	= "%s %s %s %s %s %s %s %s E %3.0lf%% <%.12s>\n";
 /***************************************************************************/
-char	*memprochdr	= "\n  PID MINFLT MAJFLT #SWP VSTEXT "
+char	*memprochdr	= "\n  PID MINFLT MAJFLT      VSTEXT "
 		     	  " VSIZE  RSIZE  VGROW  RGROW  %s CMD  %4d/%-4d\n";
-char 	*memproclina	= "%5d %s %s %s %s %s %s %s %s %3.0lf%% "
-	                  "%.14s\n";
-char 	*memprocline	= "%s %s %s                                  "
-			  "  %3.0lf%% <%.12s>\n";
-char 	*memproclinep	= "%5d %s %s %s        %s %s %s %s"
+char 	*memproclina	= "%5d %s %s      %s %s %s %s %s %3.0lf%% %.14s\n";
+char 	*memprocline	= "%s %s          0K     0K     0K     0K     0K"
 			  " %3.0lf%% <%.12s>\n";
+char 	*memproclinep	= "%5d %s %s          0K %s %s %s %s %3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*dskprochdr	= "\n  PID  READDISK AVGSIZE TOTSIZE   WRITEDISK "
 			  "AVGSIZE TOTSIZE  %s CMD  %4d/%-4d\n";
@@ -186,18 +197,18 @@ char 	*dskprocline	= " (reads&writes              %s)   "
 			  "             %3.0lf%% <%.12s>\n";
 char 	*dskproclinep	= "%5d %s %s  %s   %s %s  %s %3.0lf%% <%.12s>\n";
 /***************************************************************************/
-char	*netprochdr	= "\n  PID TCPRCV/AVSZ TCPSND/AVSZ UDPRCV/AVSZ"
-			  " UDPSND/AVSZ RAWRS  %s CMD  %4d/%-4d\n";
-char 	*netproclina	= "%5d %s/%s %s/%s %s/%s %s/%s %2llu %2llu "
+char	*netprochdr	= "\n  PID TCPRCV AVSZ|TCPSND AVSZ|UDPRCV AVSZ"
+			  "|UDPSND AVSZ|RAWRS| %s CMD  %4d/%-4d\n";
+char 	*netproclina	= "%5d %s %s|%s %s|%s %s|%s %s|%2llu %2llu|"
 			  "%3.0lf%% %.14s\n";
 char 	*netprocline	= "                                        "
 			  "              %3.0lf%% <%.12s>\n";
-char 	*netproclinep	= "%5d %s/%s %s/%s %s/%s %s/%s %2llu %2llu "
+char 	*netproclinep	= "%5d %s %s|%s %s|%s %s|%s %s|%2llu %2llu|"
 			  "%3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*varprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
 	                  "ST EXC  S  %s CMD  %4d/%-4d\n";
-char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %c%c      %c %3.0lf%% "
+char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %c%c   -  %c %3.0lf%% "
 	                  "%.14s\n";
 char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %c%c %3ld  E %3.0lf%% "
 			  "<%.12s>\n";
@@ -209,9 +220,9 @@ char 	*cmdprocline	= "%3.0lf%% <%.66s>\n";
 /***************************************************************************/
 char	*schprochdr	= "\n  PID  TGID  THR  POLICY NICE PRIO RTPRIO "
 			  "CURCPU ST EXC   S  %s CMD  %4d/%-4d\n";
-char 	*schproclina	= "%5d %5d %4d  %-6s %4d %4d %6d %6d %-2s       "
+char 	*schproclina	= "%5d %5d %4d  %-6s %4d %4d %6d %6d %-2s   -   "
 			  "%c %3.0lf%% %.14s\n";
-char 	*schprocline	= "                                            "
+char 	*schprocline	= "     -    0       -    -    -      -      - "
 			  "%c%c %3ld   E %3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*totuserhdr	= "\nNPROCS  SYSCPU  USRCPU  VSIZE  RSIZE  RDDSK "
@@ -271,7 +282,8 @@ totalcap(struct syscap *psc, struct sstat *sstat, struct pstat *pstat, int nact)
 ** calculate cumulative system- and user-time for all active processes
 */
 void
-pricumproc(struct pstat *pstat, int nact, int nproc, int nzomb, int nexit)
+pricumproc(struct pstat *pstat, int nact, int nproc, int nzomb, int nexit,
+                                int avgval, int nsecs)
 {
 	int 	i;
 	count_t totut, totst;
@@ -286,17 +298,18 @@ pricumproc(struct pstat *pstat, int nact, int nproc, int nzomb, int nexit)
 
 	printg(prcline, val2cpustr(totst * 1000/HZ, format1),
 			val2cpustr(totut * 1000/HZ, format2),
-			val2valstr(nproc,           format3, 7),
+			val2valstr(nproc,           format3, 7, 0, 0),
+			val2valstr(nzomb,           format4, 4, 0, 0),
 			acctfd >= 0 ?
-			          val2valstr(nexit, format4, 5) : "    ?",
-			val2valstr(nzomb,           format5, 4));
+			       val2valstr(nexit, format5, 6, avgval, nsecs):
+			       "     ?");
 }
 
 /*
 ** print a line of general figures about one process
 */
 void
-showgenproc(struct pstat *curstat, double perc)
+showgenproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	struct passwd 	*pwd;
 	char		*username, exittype;
@@ -325,10 +338,10 @@ showgenproc(struct pstat *curstat, double perc)
 			   curstat->gen.pid,
 			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			   val2memstr(curstat->mem.vgrow*1024,    format3,
-								  KBFORMAT),
-			   val2memstr(curstat->mem.rgrow*1024,    format4,
-								  KBFORMAT),
+			   val2memstr(curstat->mem.vgrow*1024,
+						format3, KBFORMAT, 0, 0),
+			   val2memstr(curstat->mem.rgrow*1024,
+						format4, KBFORMAT, 0, 0),
 			   username,
 			   curstat->gen.nthr,
 			   curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
@@ -342,18 +355,22 @@ showgenproc(struct pstat *curstat, double perc)
 			   curstat->gen.pid,
 			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			   val2memstr(curstat->mem.vgrow*1024,    format3,
-								  KBFORMAT),
-			   val2memstr(curstat->mem.rgrow*1024,    format4,
-								  KBFORMAT),
-			   val2valstr(curstat->dsk.rio,           format5, 5),
-			   val2valstr(curstat->dsk.wio,           format6, 5),
+			   val2memstr(curstat->mem.vgrow*1024,
+						format3, KBFORMAT, 0, 0),
+			   val2memstr(curstat->mem.rgrow*1024,
+						format4, KBFORMAT, 0, 0),
+			   val2valstr(curstat->dsk.rio,     format5, 5,
+			                                    avgval, nsecs),
+			   val2valstr(curstat->dsk.wio,     format6, 5,
+			                                    avgval, nsecs),
 			   val2valstr(curstat->net.tcprcv +
 			              curstat->net.udprcv +
-			              curstat->net.rawrcv,        format7, 4),
+			              curstat->net.rawrcv,  format7, 4,
+			                                    avgval, nsecs),
 			   val2valstr(curstat->net.tcpsnd +
 			              curstat->net.udpsnd +
-			              curstat->net.rawsnd,        format8, 4),
+			              curstat->net.rawsnd,  format8, 4,
+			                                    avgval, nsecs),
 			   curstat->gen.state,
 			   perc, curstat->gen.name);
 		}
@@ -405,18 +422,22 @@ showgenproc(struct pstat *curstat, double perc)
 			printg(genprocline2p,
 			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			   val2memstr(curstat->mem.vgrow*1024,    format3,
-								  KBFORMAT),
-			   val2memstr(curstat->mem.rgrow*1024,    format4,
-								  KBFORMAT),
-			   val2valstr(curstat->dsk.rio,           format5, 5),
-			   val2valstr(curstat->dsk.wio,           format6, 5),
+			   val2memstr(curstat->mem.vgrow*1024,
+						format3, KBFORMAT, 0, 0),
+			   val2memstr(curstat->mem.rgrow*1024,
+						format4, KBFORMAT, 0, 0),
+			   val2valstr(curstat->dsk.rio,     format5, 5,
+			                                    avgval, nsecs),
+			   val2valstr(curstat->dsk.wio,     format6, 5,
+			                                    avgval, nsecs),
 			   val2valstr(curstat->net.tcprcv +
 			              curstat->net.udprcv +
-			              curstat->net.rawrcv,        format7, 4),
+			              curstat->net.rawrcv,  format7, 4,
+			                                    avgval, nsecs),
 			   val2valstr(curstat->net.tcpsnd +
 			              curstat->net.udpsnd +
-			              curstat->net.rawsnd,        format8, 4),
+			              curstat->net.rawsnd,  format8, 4,
+			                                    avgval, nsecs),
 			   perc, curstat->gen.name);
 		   }
 		   else
@@ -424,7 +445,8 @@ showgenproc(struct pstat *curstat, double perc)
 			printg(genprocline2,
 			  val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			  val2cpustr(curstat->cpu.utime*1000/HZ, format2),
-			  val2valstr(curstat->dsk.rio, format3, 6),
+			  val2valstr(curstat->dsk.rio,      format3, 6,
+			                                    avgval, nsecs),
 			  perc, curstat->gen.name);
 		   }
 		}
@@ -435,12 +457,12 @@ showgenproc(struct pstat *curstat, double perc)
 ** print a line of memory-figures about one process
 */
 void
-showmemproc(struct pstat *curstat, double perc)
+showmemproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char	format1[16], format2[16];
 	char	format3[16], format4[16];
 	char	format5[16], format6[16];
-	char	format7[16], format8[16];
+	char	format7[16];
 
 	if (curstat->gen.state != 'E')
 	{
@@ -449,14 +471,15 @@ showmemproc(struct pstat *curstat, double perc)
 		*/
 		printg(memproclina,
 		  curstat->gen.pid,
-		  val2valstr(curstat->mem.minflt,      format1, 6),
-		  val2valstr(curstat->mem.majflt,      format2, 6),
-		  val2valstr(curstat->mem.iosw,        format3, 4),
-		  val2memstr(curstat->mem.shtext*1024, format4, KBFORMAT),
-		  val2memstr(curstat->mem.vmem  *1024, format5, KBFORMAT),
-		  val2memstr(curstat->mem.rmem  *1024, format6, KBFORMAT),
-		  val2memstr(curstat->mem.vgrow *1024, format7, KBFORMAT),
-		  val2memstr(curstat->mem.rgrow *1024, format8, KBFORMAT),
+		  val2valstr(curstat->mem.minflt,      format1, 6,
+			                               avgval, nsecs),
+		  val2valstr(curstat->mem.majflt,      format2, 6,
+			                               avgval, nsecs),
+		  val2memstr(curstat->mem.shtext*1024, format3, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.vmem  *1024, format4, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.rmem  *1024, format5, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.vgrow *1024, format6, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.rgrow *1024, format7, KBFORMAT, 0, 0),
 		  perc, curstat->gen.name);
 	}
 	else
@@ -468,13 +491,14 @@ showmemproc(struct pstat *curstat, double perc)
 	   {
 		printg(memproclinep,
 		  curstat->gen.pid,
-		  val2valstr(curstat->mem.minflt,      format1, 6),
-		  val2valstr(curstat->mem.majflt,      format2, 6),
-		  val2valstr(curstat->mem.iosw,        format3, 4),
-		  val2memstr(curstat->mem.vmem  *1024, format5, KBFORMAT),
-		  val2memstr(curstat->mem.rmem  *1024, format6, KBFORMAT),
-		  val2memstr(curstat->mem.vgrow *1024, format7, KBFORMAT),
-		  val2memstr(curstat->mem.rgrow *1024, format8, KBFORMAT),
+		  val2valstr(curstat->mem.minflt,      format1, 6,
+			                               avgval, nsecs),
+		  val2valstr(curstat->mem.majflt,      format2, 6,
+			                               avgval, nsecs),
+		  val2memstr(curstat->mem.vmem  *1024, format3, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.rmem  *1024, format4, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.vgrow *1024, format5, KBFORMAT, 0, 0),
+		  val2memstr(curstat->mem.rgrow *1024, format6, KBFORMAT, 0, 0),
 		  perc, curstat->gen.name);
 	   }
 	   else
@@ -485,9 +509,10 @@ showmemproc(struct pstat *curstat, double perc)
 			printg("%5d ", curstat->gen.pid);
 
 		printg(memprocline,
-			  val2valstr(curstat->mem.minflt,     format1, 6),
-			  val2valstr(curstat->mem.majflt,     format2, 6),
-			  val2valstr(curstat->mem.iosw,       format3, 4),
+			  val2valstr(curstat->mem.minflt,     format1, 6,
+			                                      avgval, nsecs),
+			  val2valstr(curstat->mem.majflt,     format2, 6,
+			                                      avgval, nsecs),
 			  perc, curstat->gen.name);
 	   }
 	}
@@ -497,7 +522,7 @@ showmemproc(struct pstat *curstat, double perc)
 ** print a line of disk-figures about one process
 */
 void
-showdskproc(struct pstat *curstat, double perc)
+showdskproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char	format1[16], format2[16], format3[16];
 	char	format4[16], format5[16], format6[16];
@@ -515,12 +540,16 @@ showdskproc(struct pstat *curstat, double perc)
 		*/
 		printg(dskproclina,
 		  curstat->gen.pid,
-		  val2valstr(curstat->dsk.rio,      	format1, 9),
-		  val2valstr(avgrsz,  			format2, 7),
-		  val2memstr(curstat->dsk.rsz * 512LL,	format3, KBFORMAT),
-		  val2valstr(curstat->dsk.wio,      	format4, 9),
-		  val2valstr(avgwsz,  			format5, 7),
-		  val2memstr(curstat->dsk.wsz * 512LL,	format6, KBFORMAT),
+		  val2valstr(curstat->dsk.rio,      	format1, 9,
+			                                      avgval, nsecs),
+		  val2valstr(avgrsz,  			format2, 7, 0, 0),
+		  val2memstr(curstat->dsk.rsz*512LL,	format3, KBFORMAT,
+			                                      avgval, nsecs),
+		  val2valstr(curstat->dsk.wio,      	format4, 9,
+			                                      avgval, nsecs),
+		  val2valstr(avgwsz,  			format5, 7, 0, 0),
+		  val2memstr(curstat->dsk.wsz*512LL,	format6, KBFORMAT,
+			                                      avgval, nsecs),
 		  perc, curstat->gen.name);
 	}
 	else
@@ -532,12 +561,16 @@ showdskproc(struct pstat *curstat, double perc)
 	   {
 		printg(dskproclinep,
 		  curstat->gen.pid,
-		  val2valstr(curstat->dsk.rio,      	format1, 9),
-		  val2valstr(avgrsz,  			format2, 7),
-		  val2memstr(curstat->dsk.rsz * 512LL,	format3, KBFORMAT),
-		  val2valstr(curstat->dsk.wio,      	format4, 9),
-		  val2valstr(avgwsz,  			format5, 7),
-		  val2memstr(curstat->dsk.wsz * 512LL,	format6, KBFORMAT),
+		  val2valstr(curstat->dsk.rio,      	format1, 9,
+			                                      avgval, nsecs),
+		  val2valstr(avgrsz,  			format2, 7, 0, 0),
+		  val2memstr(curstat->dsk.rsz * 512LL,	format3, KBFORMAT,
+			                                      avgval, nsecs),
+		  val2valstr(curstat->dsk.wio,      	format4, 9,
+			                                      avgval, nsecs),
+		  val2valstr(avgwsz,  			format5, 7, 0, 0),
+		  val2memstr(curstat->dsk.wsz * 512LL,	format6, KBFORMAT,
+			                                      avgval, nsecs),
 		  perc, curstat->gen.name);
 	   }
 	   else
@@ -548,7 +581,7 @@ showdskproc(struct pstat *curstat, double perc)
 			printg("%5d ", curstat->gen.pid);
 
 		printg(dskprocline,
-		  	val2valstr(curstat->dsk.rio, format1, 9),
+		  	val2valstr(curstat->dsk.rio, format1, 9, avgval, nsecs),
 			perc, curstat->gen.name);
 	   }
 	}
@@ -558,7 +591,7 @@ showdskproc(struct pstat *curstat, double perc)
 ** print a line of network-figures about one process
 */
 void
-shownetproc(struct pstat *curstat, double perc)
+shownetproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char	format1[16], format2[16], format3[16], format4[16];
 	char	format5[16], format6[16], format7[16], format8[16];
@@ -580,14 +613,18 @@ shownetproc(struct pstat *curstat, double perc)
 		*/
 		printg(netproclina,
 		  curstat->gen.pid,
-		  val2valstr(curstat->net.tcprcv,      format1,  6),
-		  val2valstr(avgtcpr,                  format2,  4),
-		  val2valstr(curstat->net.tcpsnd,      format3,  6),
-		  val2valstr(avgtcps,                  format4,  4),
-		  val2valstr(curstat->net.udprcv,      format5,  6),
-		  val2valstr(avgudpr,                  format6,  4),
-		  val2valstr(curstat->net.udpsnd,      format7,  6),
-		  val2valstr(avgudps,                  format8,  4),
+		  val2valstr(curstat->net.tcprcv,      format1,  6,
+			                                      avgval, nsecs),
+		  val2valstr(avgtcpr,                  format2,  4, 0, 0),
+		  val2valstr(curstat->net.tcpsnd,      format3,  6,
+			                                      avgval, nsecs),
+		  val2valstr(avgtcps,                  format4,  4, 0, 0),
+		  val2valstr(curstat->net.udprcv,      format5,  6,
+			                                      avgval, nsecs),
+		  val2valstr(avgudpr,                  format6,  4, 0, 0),
+		  val2valstr(curstat->net.udpsnd,      format7,  6,
+			                                      avgval, nsecs),
+		  val2valstr(avgudps,                  format8,  4, 0, 0),
 		  curstat->net.rawrcv,
 		  curstat->net.rawsnd,
 		  perc, curstat->gen.name);
@@ -601,14 +638,18 @@ shownetproc(struct pstat *curstat, double perc)
 	   	{
 			printg(netproclinep,
 		  	curstat->gen.pid,
-		  	val2valstr(curstat->net.tcprcv,      format1,  6),
-		  	val2valstr(avgtcpr,                  format2,  4),
-		  	val2valstr(curstat->net.tcpsnd,      format3,  6),
-		  	val2valstr(avgtcps,                  format4,  4),
-		  	val2valstr(curstat->net.udprcv,      format5,  6),
-		  	val2valstr(avgudpr,                  format6,  4),
-		  	val2valstr(curstat->net.udpsnd,      format7,  6),
-		  	val2valstr(avgudps,                  format8,  4),
+		  	val2valstr(curstat->net.tcprcv,      format1, 6,
+			                                      avgval, nsecs),
+		  	val2valstr(avgtcpr,                  format2, 4, 0, 0),
+		  	val2valstr(curstat->net.tcpsnd,      format3, 6,
+			                                      avgval, nsecs),
+		  	val2valstr(avgtcps,                  format4, 4, 0, 0),
+		  	val2valstr(curstat->net.udprcv,      format5, 6,
+			                                      avgval, nsecs),
+		  	val2valstr(avgudpr,                  format6, 4, 0, 0),
+		  	val2valstr(curstat->net.udpsnd,      format7, 6,
+			                                      avgval, nsecs),
+		  	val2valstr(avgudps,                  format8, 4, 0, 0),
 		  	curstat->net.rawrcv,
 		  	curstat->net.rawsnd,
 		  	perc, curstat->gen.name);
@@ -629,7 +670,7 @@ shownetproc(struct pstat *curstat, double perc)
 ** print a line of total resource consumption per user
 */
 void
-showtotuser(struct pstat *curstat, double perc)
+showtotuser(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	struct passwd 	*pwd;
 	char		numuser[16], *username;
@@ -654,19 +695,23 @@ showtotuser(struct pstat *curstat, double perc)
 	}
 
 	printg(totuserlin,
-		val2valstr(curstat->gen.pid,           format1, 6),
+		val2valstr(curstat->gen.pid,           format1, 6, 0, 0),
 		val2cpustr(curstat->cpu.stime*1000/HZ, format2),
 		val2cpustr(curstat->cpu.utime*1000/HZ, format3),
-		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT),
-		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT),
-		val2valstr(curstat->dsk.rio,           format6, 5),
-		val2valstr(curstat->dsk.wio,           format7, 5),
+		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT, 0, 0),
+		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT, 0, 0),
+		val2valstr(curstat->dsk.rio,           format6, 5,
+								avgval, nsecs),
+		val2valstr(curstat->dsk.wio,           format7, 5,
+								avgval, nsecs),
 	   	val2valstr(curstat->net.tcprcv +
 		           curstat->net.udprcv +
-			   curstat->net.rawrcv,        format8, 4),
+			   curstat->net.rawrcv,        format8, 4,
+								avgval, nsecs),
 		val2valstr(curstat->net.tcpsnd +
 			   curstat->net.udpsnd +
-			   curstat->net.rawsnd,        format9, 4),
+			   curstat->net.rawsnd,        format9, 4,
+								avgval, nsecs),
 		perc, username);
 }
 
@@ -674,7 +719,7 @@ showtotuser(struct pstat *curstat, double perc)
 ** print a line of total resource consumption per program (process name)
 */
 void
-showtotproc(struct pstat *curstat, double perc)
+showtotproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char		format1[16], format2[16];
 	char		format3[16], format4[16];
@@ -683,19 +728,23 @@ showtotproc(struct pstat *curstat, double perc)
 	char		format9[16];
 
 	printg(totproclin,
-		val2valstr(curstat->gen.pid,           format1, 6),
+		val2valstr(curstat->gen.pid,           format1, 6, 0, 0),
 		val2cpustr(curstat->cpu.stime*1000/HZ, format2),
 		val2cpustr(curstat->cpu.utime*1000/HZ, format3),
-		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT),
-		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT),
-		val2valstr(curstat->dsk.rio,           format6, 5),
-		val2valstr(curstat->dsk.wio,           format7, 5),
+		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT, 0, 0),
+		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT, 0, 0),
+		val2valstr(curstat->dsk.rio,           format6, 5,
+								avgval, nsecs),
+		val2valstr(curstat->dsk.wio,           format7, 5,
+								avgval, nsecs),
 	   	val2valstr(curstat->net.tcprcv +
 		           curstat->net.udprcv +
-			   curstat->net.rawrcv,        format8, 4),
+			   curstat->net.rawrcv,        format8, 4,
+								avgval, nsecs),
 		val2valstr(curstat->net.tcpsnd +
 			   curstat->net.udpsnd +
-			   curstat->net.rawsnd,        format9, 4),
+			   curstat->net.rawsnd,        format9, 4,
+								avgval, nsecs),
 		perc, curstat->gen.name);
 }
 
@@ -703,7 +752,7 @@ showtotproc(struct pstat *curstat, double perc)
 ** print a line of (more or less) static info about one process
 */
 void
-showvarproc(struct pstat *curstat, double perc)
+showvarproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char		format1[16], format2[16];
 	struct passwd 	*pwd;
@@ -783,7 +832,7 @@ showvarproc(struct pstat *curstat, double perc)
 ** print a line containing the (almost) full command line
 */
 void
-showcmdproc(struct pstat *curstat, double perc)
+showcmdproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	/*
 	** show process-info
@@ -824,7 +873,7 @@ showcmdproc(struct pstat *curstat, double perc)
 #define SCHED_RR                2
 
 void
-showschproc(struct pstat *curstat, double perc)
+showschproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 {
 	char	*pol, exittype;
 
@@ -953,7 +1002,7 @@ int
 priproc(struct pstat *pstat, int firstproc, int lastproc, int curline,
 	int curlist, int totlist, char *sortorder,
 	struct syscap *sb, char showtype,
-	struct selection *sel, int screen)
+	struct selection *sel, int screen, int nsecs, int avgval)
 {
 	register int		i;
 	register struct pstat	*curstat;
@@ -1083,39 +1132,39 @@ priproc(struct pstat *pstat, int firstproc, int lastproc, int curline,
 		switch (showtype)
 		{
 		   case MPROCGEN:
-			showgenproc(curstat, perc);
+			showgenproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCMEM:
-			showmemproc(curstat, perc);
+			showmemproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCDSK:
-			showdskproc(curstat, perc);
+			showdskproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCNET:
-			shownetproc(curstat, perc);
+			shownetproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCVAR:
-			showvarproc(curstat, perc);
+			showvarproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCARG:
-			showcmdproc(curstat, perc);
+			showcmdproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MPROCSCH:
-			showschproc(curstat, perc);
+			showschproc(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MCUMUSER:
-			showtotuser(curstat, perc);
+			showtotuser(curstat, perc, nsecs, avgval);
 			break;
 
 		   case MCUMPROC:
-			showtotproc(curstat, perc);
+			showtotproc(curstat, perc, nsecs, avgval);
 			break;
 		}
 
@@ -1130,7 +1179,7 @@ priproc(struct pstat *pstat, int firstproc, int lastproc, int curline,
 ** print the system-wide statistics
 */
 int
-prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
+prisyst(struct sstat *sstat, int curline, int nsecs, int avgval, int fixedhead, 
 		int maxcpulines, int maxdsklines, int maxintlines)
 {
 	register int	i, lin;
@@ -1210,27 +1259,49 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 	** MEMORY statistics
 	*/
 	printg("MEM | tot  %s | free  %s | cache %s | buff  %s | slab  %s |\n",
-		val2memstr(sstat->mem.physmem   * pgsz, format1, MBFORMAT),
-		val2memstr(sstat->mem.freemem   * pgsz, format2, MBFORMAT),
-		val2memstr(sstat->mem.cachemem  * pgsz, format3, MBFORMAT),
-		val2memstr(sstat->mem.buffermem * pgsz, format4, MBFORMAT),
-		val2memstr(sstat->mem.slabmem   * pgsz, format5, MBFORMAT));
+		val2memstr(sstat->mem.physmem   * pgsz, format1, MBFORMAT,0,0),
+		val2memstr(sstat->mem.freemem   * pgsz, format2, MBFORMAT,0,0),
+		val2memstr(sstat->mem.cachemem  * pgsz, format3, MBFORMAT,0,0),
+		val2memstr(sstat->mem.buffermem * pgsz, format4, MBFORMAT,0,0),
+		val2memstr(sstat->mem.slabmem   * pgsz, format5, MBFORMAT,0,0));
 
 	curline++;
 
 	/*
 	** SWAP statistics
 	*/
-	printg("SWP | tot  %s | free  %s | vmcom %s | "
-	       "swin %s | swout %s |\n",
-		val2memstr(sstat->mem.totswap   * pgsz, format1, MBFORMAT),
-		val2memstr(sstat->mem.freeswap  * pgsz, format2, MBFORMAT),
-		val2memstr(sstat->mem.committed * pgsz, format3, MBFORMAT),
-		val2valstr(sstat->mem.swins,            format4, 7),
-		val2valstr(sstat->mem.swouts,           format5, 6));
+	printg("SWP | tot  %s | free  %s |              "
+	       "| vmcom %s | vmlim %s |\n",
+		val2memstr(sstat->mem.totswap   * pgsz, format1, MBFORMAT,0,0),
+		val2memstr(sstat->mem.freeswap  * pgsz, format2, MBFORMAT,0,0),
+		val2memstr(sstat->mem.committed * pgsz, format3, MBFORMAT,0,0),
+		val2memstr(sstat->mem.commitlim * pgsz, format4, MBFORMAT,0,0));
 
 	curline++;
 
+	/*
+	** PAGING statistics
+	*/
+	if (fixedhead             ||
+	    sstat->mem.pgscans    ||
+	    sstat->mem.allocstall ||
+	    sstat->mem.swins      ||
+	    sstat->mem.swouts       )
+	{
+		printg("PAG | scan %s | stall %s |              "
+		       "| swin %s | swout %s |\n",
+			val2valstr(sstat->mem.pgscans,    format1, 6,
+								avgval, nsecs),
+			val2valstr(sstat->mem.allocstall, format2, 6,
+								avgval, nsecs),
+			val2valstr(sstat->mem.swins,      format3, 7,
+								avgval, nsecs),
+			val2valstr(sstat->mem.swouts,     format4, 6,
+								avgval, nsecs));
+
+		curline++;
+	}
+
 	/*
 	** DISK statistics
 	**
@@ -1250,8 +1321,10 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 		           " write %s | avio %4.0lf ms |\n",
 		    	sstat->xdsk.xdsk[i].name,
 			(double)(sstat->xdsk.xdsk[i].io_ms * 100.0 / mstot),
-		    	val2valstr(sstat->xdsk.xdsk[i].nread,  format1, 7),
-		    	val2valstr(sstat->xdsk.xdsk[i].nwrite, format2, 6),
+		    	val2valstr(sstat->xdsk.xdsk[i].nread,  format1, 7,
+								avgval, nsecs),
+		    	val2valstr(sstat->xdsk.xdsk[i].nwrite, format2, 6,
+								avgval, nsecs),
 		    	iotot ? (double)(sstat->xdsk.xdsk[i].io_ms/iotot):0.0);
 
 		    curline++;
@@ -1267,10 +1340,14 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 	{
 		printg("NET |   transport | tcpi %s | tcpo %s |"
 		       " udpi %s | udpo %s |\n",
-			val2valstr(sstat->net.tcpin,   format1, 7),
-			val2valstr(sstat->net.tcpout,  format2, 7),
-			val2valstr(sstat->net.udpin,   format3, 7),
-			val2valstr(sstat->net.udpout,  format4, 7));
+			val2valstr(sstat->net.tcpin,  format1, 7,
+								avgval, nsecs),
+			val2valstr(sstat->net.tcpout, format2, 7,
+								avgval, nsecs),
+			val2valstr(sstat->net.udpin,  format3, 7,
+								avgval, nsecs),
+			val2valstr(sstat->net.udpout, format4, 7,
+								avgval, nsecs));
 
 		curline++;
 	}
@@ -1279,10 +1356,14 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 	{
 		printg("NET |     network | ipi %s | ipo %s | ipfrw %s |"
 		       " deliv %s |\n",
-			val2valstr(sstat->net.ipin,    format1, 8),
-			val2valstr(sstat->net.ipout,   format2, 8),
-			val2valstr(sstat->net.ipfrw,   format3, 6),
-			val2valstr(sstat->net.ipindel, format4, 6));
+			val2valstr(sstat->net.ipin,    format1, 8,
+								avgval, nsecs),
+			val2valstr(sstat->net.ipout,   format2, 8,
+								avgval, nsecs),
+			val2valstr(sstat->net.ipfrw,   format3, 6,
+								avgval, nsecs),
+			val2valstr(sstat->net.ipindel, format4, 6,
+								avgval, nsecs));
 
 		curline++;
 	}
@@ -1292,13 +1373,50 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 		if (sstat->intf.intf[i].rpack ||
 		    sstat->intf.intf[i].spack || fixedhead)
 		{
+			count_t ival   = sstat->intf.intf[i].rbyte/128/nsecs;
+			count_t oval   = sstat->intf.intf[i].sbyte/128/nsecs;
+
+			char    *iform = "Kbps";
+			char    *oform = "Kbps";
+
+			if (ival >= 10000)
+			{
+				if (ival >= 100000)
+				{
+					ival /= 1024*1024;
+					iform = "Gbps";
+				}
+				else
+				{
+					ival /=1024;
+					iform = "Mbps";
+				}
+			}
+
+			if (oval >= 1000)
+			{
+				if (oval >= 1000000)
+				{
+					oval /= 1024*1024;
+					oform = "Gbps";
+				}
+				else
+				{
+					oval /=1024;
+					oform = "Mbps";
+				}
+			}
+
 			printg("NET | dev %7.7s | pcki %s | pcko %s |"
-			       " in %sK | out %sK |\n",
+			       " in %4lld %s | out %3lld %s |\n",
 			  sstat->intf.intf[i].name,
-			  val2valstr(sstat->intf.intf[i].rpack, format1, 7),
-			  val2valstr(sstat->intf.intf[i].spack, format2, 7),
-			  val2valstr(sstat->intf.intf[i].rbyte/1024,format3,8),
-			  val2valstr(sstat->intf.intf[i].sbyte/1024,format4,7));
+			  val2valstr(sstat->intf.intf[i].rpack,
+						format1, 7, avgval, nsecs),
+			  val2valstr(sstat->intf.intf[i].spack,
+						format2, 7, avgval, nsecs),
+			  ival, iform,
+			  oval, oform);
+
 			curline++;
 			lin++;
 		}
