@@ -30,6 +30,10 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: various.c,v $
+** Revision 1.13  2006/02/07 08:27:21  gerlof
+** Add possibility to show counters per second.
+** Modify presentation of CPU-values.
+**
 ** Revision 1.12  2005/10/31 12:26:09  gerlof
 ** Modified date-format to yyyy/mm/dd.
 **
@@ -68,7 +72,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: various.c,v 1.12 2005/10/31 12:26:09 gerlof Exp $";
+static const char rcsid[] = "$Id: various.c,v 1.13 2006/02/07 08:27:21 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -175,16 +179,27 @@ daysecs(time_t itime)
 ** number of positions; if the value does not fit, it will be formatted to
 ** exponent-notation (as short as possible, so not via the standard printf-
 ** formatters %f or %e). The offered string should have a length of width+1.
+** The value might even be printed as an average for the interval-time.
 */
 char *
-val2valstr(count_t value, char *strvalue, int width)
+val2valstr(count_t value, char *strvalue, int width, int avg, int nsecs)
 {
-	count_t	maxval = pow(10.0, width) - 1, remain = 0;
-	int	exp = 0;
+	count_t	maxval, remain = 0;
+	int	exp     = 0;
+	char	*suffix = "";
+
+	if (avg)
+	{
+		value  = (value + (nsecs/2)) / nsecs;     /* rounded value */
+		width  = width - 2;	/* subtract two positions for '/s' */
+		suffix = "/s";
+	}
+
+	maxval = pow(10.0, width) - 1;
 
 	if (value < maxval)
 	{
-		sprintf(strvalue, "%*lld", width, value);
+		sprintf(strvalue, "%*lld%s", width, value, suffix);
 	}
 	else
 	{
@@ -193,7 +208,7 @@ val2valstr(count_t value, char *strvalue, int width)
 			/*
 			** cannot avoid ignoring width
 			*/
-			sprintf(strvalue, "%lld", value);
+			sprintf(strvalue, "%lld%s", value, suffix);
 		}
 		else
 		{
@@ -214,7 +229,8 @@ val2valstr(count_t value, char *strvalue, int width)
 			if (remain >= 5)
 				value++;
 
-			sprintf(strvalue, "%*llde%d", width, value, exp);
+			sprintf(strvalue, "%*llde%d%s",
+					width, value, exp, suffix);
 		}
 	}
 
@@ -226,7 +242,7 @@ val2valstr(count_t value, char *strvalue, int width)
 ** to an ascii-string of 7 positions in milliseconds or minute-seconds or
 ** hours-minutes, stored in strvalue (at least 8 positions).
 */
-#define	MAXMSEC		(count_t)10000
+#define	MAXMSEC		(count_t)100000
 #define	MAXSEC		(count_t)60000
 
 char *
@@ -234,7 +250,7 @@ val2cpustr(count_t value, char *strvalue)
 {
 	if (value < MAXMSEC)
 	{
-		sprintf(strvalue, "%4lld ms", value);
+		sprintf(strvalue, "%3lld.%02llds", value/1000, value%1000/10);
 	}
 	else
 	{
@@ -277,10 +293,13 @@ val2cpustr(count_t value, char *strvalue)
 #define	MAXMBYTE	ONEMBYTE*999L
 
 char *
-val2memstr(count_t value, char *strvalue, int pformat)
+val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 {
 	char 	aformat;	/* advised format		*/
 	count_t	verifyval;
+	char	*suffix = "";
+	int	basewidth = 6;
+
 
 	/*
 	** notice that the value can be negative, in which case the
@@ -292,6 +311,17 @@ val2memstr(count_t value, char *strvalue, int pformat)
 	else
 		verifyval =  value;
 
+	/*
+	** verify if printed value is required per second (average) or total
+	*/
+	if (avgval)
+	{
+		value     /= nsecs;
+		verifyval *= 100;
+		basewidth -= 2;
+		suffix     = "/s";
+	}
+	
 	/*
 	** determine which format will be used on bases of the value itself
 	*/
@@ -315,19 +345,23 @@ val2memstr(count_t value, char *strvalue, int pformat)
 	switch (aformat)
 	{
 	   case	ANYFORMAT:
-		sprintf(strvalue, "%6lld", value);
+		sprintf(strvalue, "%*lld%s",
+				basewidth, value, suffix);
 		break;
 
 	   case	KBFORMAT:
-		sprintf(strvalue, "%5lldK", value/ONEKBYTE);
+		sprintf(strvalue, "%*lldK%s",
+				basewidth-1, value/ONEKBYTE, suffix);
 		break;
 
 	   case	MBFORMAT:
-		sprintf(strvalue, "%5.1lfM", (double)((double)value/ONEMBYTE)); 
+		sprintf(strvalue, "%*.1lfM%s",
+			basewidth-1, (double)((double)value/ONEMBYTE), suffix); 
 		break;
 
 	   case	GBFORMAT:
-		sprintf(strvalue, "%5.1lfG", (double)((double)value/ONEGBYTE));
+		sprintf(strvalue, "%*.1lfG%s",
+			basewidth-1, (double)((double)value/ONEGBYTE), suffix);
 		break;
 
 	   default:
